#	Illustrative figure for
#	Engler and Nielsen:
#	Empirical process of autoregressive residuals
#	5 May 2009.

require(stats)

Drive	<- "e:/data/fultonfish/"
Width	<- 7
Height	<- 7
 
filename<- "Fig"
postscript(file=paste(Drive,filename,".eps",sep=""),horizontal=FALSE,
	family="Times",onefile=FALSE,
	bg="transparent",fg="black",
	paper="special",width=Width,height=Height)

par(mfrow=c(2,2))
par(mar=c(4.5,4.5,2,0.5),oma=c(0,0,0,0))
par(font.main=1)
par(las=1)
par(cex=1.2)
par(mex=0.7)


#	FultonFish
DataFile	<-	paste(Drive,"Fulton.dat",sep="")  
VarName		<-	c("Monday","Tuesday","Wednesday","Thursday","Date","Stormy","Mixed",
					"p","q","Rainy","Cold","Windspeed","Windspeed2")
n		<- 	111
#	Select p
SelVar	<-	c(8)

Data	<- read.table(file=DataFile,col.names=VarName,na.strings=".")
X	<- as.matrix(Data[1:n,SelVar])

p_	<- X[1:n-1]
p	<- X[2:n]

# OLS regression, AR1 with intercept 
OLS <- lsfit(x=p_, y=p)
res <-OLS$residuals
stdres <- (res-mean(res))/sd(res)

# 	Empirical distribution statistics
Ores <- sort(stdres)		# order statistics
Fhat <- seq(1,n-1)/n		# Fhat: values of empirical distribution
Onor <- qnorm(Fhat)			# normal quantiles
Fnor <- pnorm(Ores)			# F: values of reference distribution
dnor <- dnorm(Ores)			# f: values of reference density


#############################
## Plots of data + residuals

time<- seq(1,n,by=1)

plot(time,X,type="l",xlab="days",ylab="log price")
title(	main="(a) Prices",
		adj=0)

N	<- n-1
time<- seq(1,N,by=1)

plot(time,stdres,type="l",
		xlab="days",ylab="residuals")
title(	main="(b) Residuals",
		adj=0)

	
#######################
## PP + data

## 1. PP data

xLower	<- 0;	xUpper <- 1
yLower	<- 0;	yUpper <- 1
plot(c(xLower,xUpper),c(yLower,yUpper),type="l",lty=2,
		xlab="Normal Distribution Function",
		ylab="Empirical Distribution Function")
title( 	main="(c) PP-plot",
		adj=0)
lines(Fnor,Fhat)

## 2. PP pointwise cf

L	<- 1000
xx	<- seq(xLower,xUpper,length=L)
qq	<- qnorm(xx)
dx	<- dnorm(qq)
qq[1] <- 0; qq[L] <- 0;
Fse	<- xx*(1-xx)-dx^2*(qq^2/2+1)

lines(xx,pmin(xx+2*sqrt(Fse/N),1),col="blue")
lines(xx,pmax(xx-2*sqrt(Fse/N),0),col="blue")

##	3. PP KS

lines(xx,pmin(xx+0.903/sqrt(N),1),col="green")
lines(xx,pmax(xx-0.903/sqrt(N),0),col="green")
lines(c(0,0),c(0,0+0.903/sqrt(N)),col="green")
lines(c(1,1),c(1,1-0.903/sqrt(N)),col="green")

##	4. PP Anderson-Darling

a	<- 0.45;
#	Compute critical value from Gamma approximation.
#	First evaluate response surface for mean and var.
e	<- 1.969+1.405*a		  -0.04406/a+0.0008920/a^2+0.00028/(0.5-a)
v   <- 0.365-0.291*a+0.141*a^2						  -0.00001/(0.5-a)
qPP_AD	<- qgamma(0.95,shape=e^2/v,scale=v/e)
q	<- qPP_AD
xx	<- seq(0.5-a,0.5+a,length=L)
qq	<- qnorm(xx)
dx	<- dnorm(qq)
Fse	<- xx*(1-xx)-dx^2*(qq^2/2+1)
lines(xx,xx+q*sqrt(Fse/N),col="red")
lines(xx,xx-q*sqrt(Fse/N),col="red")
lines((0.5-a)*c(1,1),0.5-a+q*sqrt(Fse[1]/N)*c(1,-1),col="red")
lines((0.5+a)*c(1,1),0.5+a+q*sqrt(Fse[L]/N)*c(1,-1),col="red")

#legend(0,1,lty=1,
#		legend=c("Pointwise bands","Kolmogorov-Smirnov: supremum","Anderson-Darling: supremum+standardise"),
#		col=c("blue","green","red"))

#######################
## QQ + data

xLower	<- min(Onor);	xUpper <- max(Onor)
yLower	<- min(Ores);	yUpper <- max(Ores)

plot(c(xLower,xUpper),c(xLower,xUpper),lty=2,type="l",
		xlim=c(xLower,xUpper),ylim=c(yLower,yUpper),
		xlab="Normal Quantiles",
		ylab="Empirical Quantiles")
title(	main="(d) QQ-plot",
		adj=0)

##	1. QQ - data

lines(Onor,Ores)

##	2. QQ - Pointwise

L	<- 100
xx	<- seq(xLower,xUpper,length=L)
Fx	<- pnorm(xx)
dx	<- dnorm(xx)
Fse	<- Fx*(1-Fx)-dx^2*(xx^2/2+1)
Ose	<- Fse/dx^2

lines(xx,xx+2*sqrt(Ose/N),col="blue")
lines(xx[9:L],(xx-2*sqrt(Ose/N))[9:L],col="blue")
lines(c(xLower,xUpper),c(yLower,yLower),lty=0,lwd=10,col="blue")

##	3. QQ - Kolmogorov Smirnov-type

a  	<- 0.45			
##	Compute critical value from Gamma approximation.
##	First evaluate response surface for mean and var.
e	<- 1.672-1.583*a+4.904*a^2-0.02773/a+0.03722/(0.5-a)-0.0000677/(0.5-a)^2
v   <- 0.135+0.053*a-1.293*a^2			+0.02743/(0.5-a)-0.0000151/(0.5-a)^2
qQQ_KS	<- qgamma(0.95,shape=e^2/v,scale=v/e)
q	<- qQQ_KS
x <- qnorm(0.5+a)
lines(c(-x,x),c(-x,x)+q/sqrt(N),col="green")
lines(c(-x,x),c(-x,x)-q/sqrt(N),col="green")
lines(-x*c(1,1),-x*c(1,1)+q/sqrt(N)*c(-1,1),col="green")
lines(+x*c(1,1),+x*c(1,1)+q/sqrt(N)*c(-1,1),col="green")

##	4. QQ - Anderson-Darling-type

a  	<- 0.45;		
##	Compute critical value from Gamma approximation.
##	First evaluate response surface for mean and var.
##	Note this is the same as for Anderson-Darling for PP plot
e	<- 1.969+1.405*a		  -0.04406/a+0.0008920/a^2+0.00028/(0.5-a)
v   <- 0.365-0.291*a+0.141*a^2						  -0.00001/(0.5-a)
qQQ_AD	<- qgamma(0.95,shape=e^2/v,scale=v/e)
q	<- qQQ_AD
xx	<- seq(qnorm(0.5-a),qnorm(0.5+a),length=L)
Fx	<- pnorm(xx)
dx	<- dnorm(xx)
Fse	<- Fx*(1-Fx)-dx^2*(xx^2/2+1)
Ose	<- Fse/dx^2
lines(xx,xx+q*sqrt(Ose/N),col="red")
lines(xx,xx-q*sqrt(Ose/N),col="red")
lines(c(xLower,xUpper),c(yLower,yLower),lty=0,lwd=10,col="red")

#legend(xLower,yUpper,lty=1,
#		legend=c("Pointwise bands","Kolmogorov-Smirnov-type: supremum","Anderson-Darling-type"),
#		col=c("blue","green","red"))

dev.off()

